#!/usr/bin/env ruby
# PLOP sample application for dumping PDF information with pCOS
#
# Id: dumper.pl,v 1.6 2010/07/15 10:34:12 rjs Exp 

require './PLOP'

begin
#    if (ARGV.length != 1)
#    raise "usage: dumper <filename>"
#    end

    plop = PLOP.new
    infilename = "PLOP-datasheet.pdf"

    docoptlist = "requiredmode=minimum"
    globaloptlist = ""
    # This is where input files live. Adjust as necessary.
    searchpath = "{../data ../../data}"

    plop.set_option(globaloptlist)

    plop.set_option("searchpath=" + searchpath)


    if ((doc = plop.open_document(infilename, docoptlist)) == -1)
        raise "ERROR: " + plop.get_errmsg()
    end

    # --------- general information (always available)

    pcosmode = plop.pcos_get_string(doc, "pcosmodename")

    printf("       File name: %s\n", plop.pcos_get_string(doc, "filename"))

    printf("     PDF version: %s\n",
        plop.pcos_get_string(doc, "pdfversionstring"))

    printf("      Encryption: %s\n",
        plop.pcos_get_string(doc, "encrypt/description"))

    recipientcount =
             plop.pcos_get_number( doc, "length:encrypt/recipients")

    if (recipientcount > 0)
        printf("Recipient groups: %d\n", recipientcount)
        printf("      Digital ID: ")

        if (pcosmode == "full")
            printf("supplied digital ID has master permissions\n")
        elsif (pcosmode == "restricted")
            printf("supplied digital ID doesn't have master permissions\n")
        else
            printf("no suitable digital ID supplied\n")
        end
    end


    printf("       Master pw: %s\n",
        (plop.pcos_get_number(doc, "encrypt/master") != 0) ? "yes" : "no")

    printf("         User pw: %s\n",
        (plop.pcos_get_number(doc, "encrypt/user") != 0) ? "yes" : "no")

    printf("    Text copying: %s\n",
        (plop.pcos_get_number(doc, "encrypt/nocopy") != 0) ? "no" : "yes")

    printf("      Linearized: %s\n",
        (plop.pcos_get_number(doc, "linearized") != 0) ? "yes" : "no")

    if (pcosmode == "minimum")
        printf("    Minimum mode: no more information available\n\n")
        return
    end

    # --------- more details (requires at least user password)

    printf("    PDF/X status: %s\n", plop.pcos_get_string(doc, "pdfx"))

    printf("    PDF/A status: %s\n", plop.pcos_get_string(doc, "pdfa"))

    printf("   PDF/UA status: %s\n", plop.pcos_get_string(doc, "pdfua"))

    printf("   PDF/VT status: %s\n", plop.pcos_get_string(doc, "pdfvt"))

    printf("        XFA data: %s\n",
        (plop.pcos_get_string(doc, "type:/Root/AcroForm/XFA") != "null" ? "yes" : "no"))
                            
    printf("      Tagged PDF: %s\n\n",
        (plop.pcos_get_number(doc, "tagged") != 0) ? "yes" : "no")

    # --------- analyze digital signatures 

    count = plop.pcos_get_number(doc, "length:signaturefields")

    0.step(count-1,1) do |i|
        sigfields = "signaturefields[" + i.to_i.to_s + "]/fullname"
        printf("     Sig. field #%d: '%s',", (i+1), plop.pcos_get_string(doc, sigfields))

        sigfields = "signaturefields[" + i.to_i.to_s + "]/visible"
        printf(" %s", plop.pcos_get_number(doc, sigfields) ? "visible" : "invisible")

        sigfields = "signaturefields[" + i.to_i.to_s + "]/sigtype"
        sigtype = plop.pcos_get_string(doc, sigfields)

        if (sigtype == "none")
            printf(", unsigned\n")
            next
        end

        printf(" %s signature", sigtype)

        if (sigtype == "certification")
            sigfields = "signaturefields[" + i.to_i.to_s + "]/permissions"
            printf(" (%s allowed)", plop.pcos_get_string(doc, sigfields))
        end

        sigfields = "signaturefields[" + i.to_i.to_s + "]/cades"
        if (plop.pcos_get_number(doc, sigfields) != 0)
            printf(", CAdES")
        end

        printf("\n")
    end

    if (count)
        printf("No. of revisions: %d\n\n", plop.pcos_get_number(doc, "revisions"))
    end

    printf("  Reader-enabled: %s\n", plop.pcos_get_string(doc, "usagerights"))

    printf("    No. of pages: %d\n", plop.pcos_get_number(doc, "length:pages"))

    printf("     Page 1 size: width=%g, height=%g\n",
        plop.pcos_get_number(doc, "pages[0]/width"),
        plop.pcos_get_number(doc, "pages[0]/height"))

    count = plop.pcos_get_number(doc, "length:fonts")
    printf("    No. of fonts: %d\n", count)

    0.step(count-1,1) do |i|
        fonts = "fonts[" + i.to_i.to_s + "]/embedded"
        if (plop.pcos_get_number(doc, fonts))
            printf("embedded ")
        else
            printf("unembedded ")
        end

        fonts = "fonts[" + i.to_i.to_s + "]/type"
        printf("%s font ", plop.pcos_get_string(doc, fonts))
        fonts = "fonts[" + i.to_i.to_s + "]/name"
        printf("%s\n", plop.pcos_get_string(doc, fonts))
    end

    printf("\n")

    plainmetadata = plop.pcos_get_number(doc, "encrypt/plainmetadata") != 0

    if (pcosmode == "restricted" && !plainmetadata &&
            plop.pcos_get_number(doc, "encrypt/nocopy") != 0)
        printf("Restricted mode: no more information available\n\n")
        return
    end

    # --------- document info keys and XMP metadata (requires master pw
    # or plaintext metadata)

    count = plop.pcos_get_number(doc, "length:/Info")

    0.step(count-1,1) do |i|
        info = "type:/Info[" + i.to_i.to_s + "]"
        objtype = plop.pcos_get_string(doc, info)
        info = "/Info[" + i.to_i.to_s + "].key"
        key = plop.pcos_get_string(doc, info)
        len = 13 - key.length()

        len.step(1,-1) do |len|
            print " "
        end
        
        print key + ": "

        # Info entries can be stored as string or name objects
        if (objtype != "string" || objtype != "name")
            info = "/Info[" + i.to_i.to_s + "]"
            print "'" + plop.pcos_get_string(doc, info) + "'" + "\n"
        else
            info = "type:/Info[" + i.to_i.to_s + "]"
            print "(" + plop.pcos_get_string(doc, info) + " object)" + "\n"
        end
    end

    printf("\n   XMP meta data: ")

    objtype = plop.pcos_get_string(doc, "type:/Root/Metadata")
    if (objtype == "stream")
        contents = plop.pcos_get_stream(doc, "", "/Root/Metadata")
        print contents.length.to_s + " bytes"

        # This demonstrates Unicode conversion
        dummy = plop.convert_to_unicode("utf8", contents, "outputformat=utf16")
        printf(" (%d Unicode characters)\n\n", dummy.length/2)
    else
        printf("not present\n\n")
    end

    plop.close_document(doc, "")
rescue PLOPException => pe
    print pe.backtrace.join("\n")
    printf("PLOP exception occurred in dumper:\n")
    print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
                        ": " + pe.get_errmsg + "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    plop.delete() if plop
end
